#ifndef DSP_H

#define DSP_BASE0 0x08000000
#define DSP_BASE1 0x10000000
#define DSP_BASE2 0x00000000
#define DSP_BASE3 0x00000000
#define DSP_BASE4 0x00000000

#define DSP_AM 0x39
#define DSP_SIZE 0x8000

typedef int int32;
typedef unsigned int u_int32;

#define TIMEOUT_PERIOD (200000)
#define DSP_RES_PERIOD 20


	/* max number of errors to log; subsequent errors are counted but not logged */
#define MAX_ERR_LOG  (50)

	/* parm1 = int error code, 	parm2 = char* error message */
	/* note that a host-specific offset may be added to parm1 here */
#define LOG_ERR(parm1, parm2) log_err((1000+parm1), parm2)

/* log contents of boot structure */
#define LOG_BOOT( ) log_boot( )				

#define DBS_RESETTING    (0)		/* DSP has not finished resetting; DBS_RESETTING must be 0 */
#define DBS_INITIALIZING (1)		/* DSP is initializing boot structure */
#define DBS_READY        (2)		/* ready to start download */
#define DBS_DOWNLOADING  (3)		/* download has begun */
#define DBS_DOWNLOAD_OK  (4)		/* the downloaded code is running */

#define ERR_FIRST_WORD   (0xA0)	/* these error codes may appear in dsp_bstate */
#define ERR_PAGE_FAILED  (0xA1)
#define ERR_BLK_SIZE     (0xA2)
#define ERR_BLK_DEST     (0xA3)

										/* host boot states (written by host) */
#define HBS_RESETTING   (0xf00ba123)	/* forces delay of start of boot */
#define HBS_IDLE        (0)				/* HBS_IDLE must be 0 */
#define HBS_READY       (0x0ad00dad)	/* causes DSP to proceed with boot */

#define PAGE_SIZE       (512)			/* number of u_int32's in a page */


#define HAND_HI			(0xff)			/* legal handshake values for req and ack */
#define HAND_LO			(0)

/*
This structure is in the VME card's dual-port RAM.
Members of this structure are written by the DSP unless noted otherwise.
  ! The first  member of boot_dpr must be host_bstate !
  ! The second member of boot_dpr must be boots      !
*/

struct boot_dpr {
  u_int32 host_bstate;				/* host boot state, written by host */
  u_int32 boots;					/* number of boots (set to 0 by host) */

  u_int32 rom_rev;					/* boot ROM revision number */
  u_int32 dip_switches;				/* ddddvvvvvvvv, d = DSP's dip switch, v = VME card's dip switch */
  u_int32 dsp_bstate;				/* DSP boot state */
  u_int32 req, ack;					/* handshake flags, ack is written by host */
  u_int32 lo_mem, hi_mem;			/* external ram available during boot */
  u_int32 words_read;				/* words of system downloaded so far */
  u_int32 ram_loops, ram_errors;	/* memory test information */
  u_int32 dummy [ 4 ];				/* fill in unused words to make 16 total */

  u_int32 zero[ 8 ];				/* zeroes (used by messaging system) */
  u_int32 page[ PAGE_SIZE ];		/* block through which system code is transferred */
};

#define EXT0_SIZE      0x40000
#define EXT1_SIZE      0x40000
#define CODE_BLOCKSIZE 0x2800
                        /* size of RAM block allocated for code       */
                        /* 0x800: boot code, 0x2000 code              */
#define MESS_BLOCKSIZE 0x2000     
                        /* size of message buffer in RAM              */
#define TRIG_BLOCKSIZE 0x2000     
                        /* size of block where trigger info is stored */
#define DATA_BLOCKSIZE (EXT0_SIZE - CODE_BLOCKSIZE - MESS_BLOCKSIZE \
                               - TRIG_BLOCKSIZE)
                       /* size of block in RAM where data is stored:   */
                       /* add EXT1_SIZE if expansion RAM card available*/

/**********************************************************************/
/*                          MESSAGES:                                 */
/**********************************************************************/

/**********************************************************************/
/* sys_generic.h:                                                     */
/*     sys_messhead.h    message header definition                    */
/*     sys_gener_mess.h  structure definitions of generic message     */
/**********************************************************************/
/* sys_messhead.h                                                     */
/*                 MESSAGE HEADER                                     */
/**********************************************************************/

#define HEADER   u_int32 ciw; \
                 u_int32 len; \
                 u_int32 dsp; \
                 u_int32 dsp_dipsw; \
                 u_int32 channel; \
                 u_int32 host_ciw; \
                 u_int32 spare[5];/* message header            */

/**********************************************************************/
/*                    Messages/Action initiated by VME CPU            */
/**********************************************************************/
#define INITIALIZE                 1
#define DISABLE_READOUT            3
#define ENABLE_READOUT             4
#define SELECT_TRIGGER_MODE        5
#define TRANSFER_DATA              6
#define SET_DAC                    9
#define SELECT_ALIGN_DELAY        11
#define LOAD_GRAYCODE_PATTERN     12
#define LOAD_CODE                 14
#define LOAD_DATA                 15
#define CALIBRATE                 16
#define SELECT_RUN_MODE           17
#define SELECT_TIME_PARAM         18
#define ECHO                      19
#define MESSAGE_TEST              20
#define GET_DSP_STATUS            21
#define VALIDATE_DATA             22
/**********************************************************************/
/*                    Messages sent from DSP to VME                   */
/**********************************************************************/
#define DATA_TRANSFER             50
#define DATA_SUMMARY              51
#define RESET_DONE                52
#define RESPONSE                  53
#define CODE_LOADED               54
#define ECHO_REPLY                55
#define TEST_MESSAGE              56
#define DSP_STATUS                57
/**********************************************************************/
/*                    Error messages sent from DSP to VME             */
/**********************************************************************/
#define FATAL_ERROR               80
#define ERROR_REPORT              81

/**********************************************************************/
/* sys_error_codes.h                                                  */
/*                    ERROR CODE DEFINITIONS                          */
/**********************************************************************/
/* fatal errors:     reported by message fatal_error                  */

#define NO_LL             1   /* low  level card not present */                
#define NO_HL             2   /* high level card not present */
#define RO_ERROR          3   /* readout error               */
#define MEM_OVERFLOW      4   /* memory overflow             */
#define INVALID_RUNMODE   5
#define INVALID_AL_DELAY  6
#define GRAYCODE_TOO_LONG 7
#define INVALID_LINK      8
#define COMMAND_UNKNOWN   9
#define FORMAT_UNKNOWN    10
#define MESSAGE_TOO_LONG  11

/* other errors:     reported by message error_report                 */

#define INVALID_DAC_VALUE         50
#define INVALID_DESTINATION_DAC   51  
#define TRIG_ERROR                52   /* error reading trigger info  */
#define NO_DET_DATA               53
#define NO_TRIG_DATA              54
#define NO_DATA                   55

/**********************************************************************/
/* sys_com_error_codes.h                                              */
/* communication error codes                                          */
/**********************************************************************/

/* these error codes moved into file sys_pub.h                        */

/**********************************************************************/
/* sys_response_codes.h                                               */
/*                        RESPONSE CODE DEFINITIONS                   */
/**********************************************************************/

#define DONE         0
#define ACCEPTED     1
#define REFUSED      2

/**********************************************************************/
/* sys_flags.h                                                        */
/*                        DEFINITIONS OF FLAGS                        */
/**********************************************************************/

#define SOB_SIG_FLAG           0
#define EOB_SIG_FLAG           1
#define TRIGGER_FLAG           2
#define DSP_DATA_READY_FLAG    3
#define DSP_STATE              4
#define ABORT_FLAG             5
#define VERBOSE_FLAG           6
#define BOOT_CODE_VERSION      7     
#define HOST_STATE             8
#define CODEVER_SYS            9
#define CODEVER_SCOM_US       10

/**********************************************************************/
/* DEFINITIONS OF DSP_STATE FLAG VALUES                               */
/*                                                                    */
/* RESET_COMPLETE = 4: reset done, boot code running,                 */
/*                        dsp is waiting for message load_code        */
/* DOWNLOAD_COMPLETE = 5: code loaded and running                     */
/* IDLE           = 1: dsp is waiting for                             */
/*                     start of burst signal or messages              */
/* EXECUTING      = 2: dsp is executing action initiated by message   */
/* BURST_LOOP     = 3: dsp is in burst-loop                           */
/*                     (reading data, waiting for end of burst signal)*/
/*                                                                    */
/*                  values 20 - 60:  states reserved for strips code  */
/*                  values 61 - 100: states reserved for pixel code   */
/**********************************************************************/

#define IDLE              1
#define EXECUTING         2
#define BURST_LOOP        3    
#define RESET_COMPLETE    4
#define DOWNLOAD_COMPLETE 5

/**********************************************************************/
/* DEFINITIONS OF HOST_STATE FLAG VALUES                              */
/**********************************************************************/
#define HS_RESETTING_DSP  0xfadedbad
#define HS_READY          0x0

/**********************************************************************/
/* RUN_MODE DEFINITIONS                                               */
/**********************************************************************/
#define EOB_TRANSFER 0
#define RT_TRANSFER  1
#define CALIB_RUN    2

/**********************************************************************/
/* OTHER DEFINITIONS                                                  */
/**********************************************************************/
#define MAX_AL_DELAY    255
#define MAX_GRAYCODE    256
#define MAX_DATA_FORMAT 10
#define DSP_LED_G       2          /* output port bit numbers */
#define DSP_LED_Y       1
#define DSP_LED_R       0
#define DSP_LED_ON      0
#define DSP_LED_OFF     1


#define SUCCESS 		0
#define ENOSIG 		1
#define EABORT 		2
#define ENOMESS 		3
#define ECONFLICT 	4
#define EBUSY			5
#define ETOOBIG		6
#define EMESSINVAL   7
#define ETIMEOUT     8

#define NUM_FLAGS		64
#define NUM_CHAN     16

#define DPRAM_SIZE     0x800


/* status codes returned in dsp_status by download system */
#define DOWN_DONE        (    0)  /* download done without errors */
#define DOWN_INCOMPLETE  (0xA00)  /* no DSP errors, but download is not done */
#define DOWN_TIMEOUT     (0xA01)  /* DSP timed out */
#define DOWN_ERROR       (0xA02)  /* Some error besides timeout */
#define MIMPORT		0
#define MEXPORT		1
#define DAV		2
#define ACK		3
#define EOM		4
#define ABORT		5


#define MTU 		1024	
#define LOOKUPS		3


typedef struct {
  int id;
  int timestamp;
  int return_add;
  int size;
} t_head;

typedef struct {
  t_head header;
  int message[MTU];
} transport;

typedef struct {
  u_int32 boot[16];         /* reserved for DSP's system download code */
  volatile int signals;
  transport packet ;
  volatile int flags[NUM_FLAGS];
} dpr;


#define MESS_TIMEOUT   (2000000)
#define PORT_OFFSET    DPRAM_SIZE

#define TMESSID()	io_base->packet.header.id
#define TTIME()	io_base->packet.header.timestamp
#define	TRETURN()	io_base->packet.header.return_add
#define TSIZE()	io_base->packet.header.size
#define FLAG(y)	io_base->flags[y]

#define DO_T_HEAD()	io_base->packet.header.id = mess_id; \
                        io_base->packet.header.timestamp = time; \
                        io_base->packet.header.return_add = iam; \
                        io_base->packet.header.size = remain

#define MCIW(x)		x->ciw
#define MSIZE(x)	x->len
#define MDEST(x)        x->dsp

#define DSP_H